/**
 *
 */
package com.paypal.hybris.core.service.impl;

import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import com.paypal.hybris.core.dao.PaymentTransactionsDao;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.AbstractOrderEntryModel;
import de.hybris.platform.core.model.order.OrderEntryModel;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.ordercancel.OrderCancelRecordsHandler;
import de.hybris.platform.ordercancel.OrderStatusChangeStrategy;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.model.ModelService;
import java.math.BigDecimal;
import java.util.ArrayList;

import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

@UnitTest
public class DefaultPaymentTransactionsServiceTest {

    private static String TRANSACTION_ID = "transactionId";
    private static String CAPTURE_ID = "captureId";
    private static String AUTHORIZATION_ID = "authorizationId";
    private static String STATUS = "status";
    private static String AMOUNT = "4";

    @Mock
    PaymentTransactionsDao transactionsDao;

    @Mock
    ModelService modelService;

    @Mock
    OrderCancelRecordsHandler orderCancelRecordsHandler;

    @Mock
    OrderStatusChangeStrategy completeCancelStatusChangeStrategy;

    @Mock
    DefaultPayPalPaymentService paypalPaymentService;

    @InjectMocks
    private DefaultPaymentTransactionsService unit;

    @Before
    public void setUp() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void updatePaymentTransactionCaptureTransactionEntryNull() {

        PaymentTransactionEntryModel entryModel = Mockito.mock(PaymentTransactionEntryModel.class);

        when(transactionsDao.getTransactionEntryById(CAPTURE_ID)).thenReturn(null);

        unit.updatePaymentTransaction(AUTHORIZATION_ID, CAPTURE_ID, STATUS, AMOUNT);

        verify(modelService, never()).save(entryModel);

    }

    @Test
    public void updatePaymentTransactionCaptureTransactionEntryNotNullTypeCapture() {

        PaymentTransactionEntryModel entryModel = Mockito.mock(PaymentTransactionEntryModel.class);

        when(transactionsDao.getTransactionEntryById(CAPTURE_ID)).thenReturn(entryModel);
        when(entryModel.getType()).thenReturn(PaymentTransactionType.CAPTURE);

        unit.updatePaymentTransaction(AUTHORIZATION_ID, CAPTURE_ID, STATUS, AMOUNT);

        verify(modelService).save(entryModel);

    }

    @Test
    public void updatePaymentTransactionCaptureTransactionEntryNotNullTypeAuthorization() {

        PaymentTransactionEntryModel entryModel = Mockito.mock(PaymentTransactionEntryModel.class);
        PaymentTransactionModel transactionModel = Mockito.mock(PaymentTransactionModel.class);
        OrderModel order = Mockito.mock(OrderModel.class);

        when(transactionsDao.getTransactionEntryById(AUTHORIZATION_ID)).thenReturn(entryModel);
        when(entryModel.getType()).thenReturn(PaymentTransactionType.AUTHORIZATION);

        when(entryModel.getPaymentTransaction()).thenReturn(transactionModel);
        when(transactionModel.getPlannedAmount()).thenReturn(BigDecimal.TEN);
        when(transactionModel.getOrder()).thenReturn(order);

        unit.updatePaymentTransaction(AUTHORIZATION_ID, CAPTURE_ID, STATUS, AMOUNT);

        verify(modelService).save(order);

    }

    @Test
    public void voidPaymentTransactionWithTransactionEntryNull() {

        PaymentTransactionEntryModel entryModel = Mockito.mock(PaymentTransactionEntryModel.class);

        when(transactionsDao.getTransactionEntryById(TRANSACTION_ID)).thenReturn(null);

        unit.voidPaymentTransaction(TRANSACTION_ID);

        verify(modelService, never()).save(entryModel);

    }

    @Test
    public void voidPaymentTransactionWithAuthorizeTypeTransactionEntryNotNull() {

        PaymentTransactionEntryModel entryModel = Mockito.mock(PaymentTransactionEntryModel.class);
        PaymentTransactionModel transactionModel = Mockito.mock(PaymentTransactionModel.class);
        PaymentTransactionEntryModel newTransactionEntryModel = Mockito.mock(PaymentTransactionEntryModel.class);
        OrderModel order = Mockito.mock(OrderModel.class);
        OrderEntryModel orderEntry = Mockito.mock(OrderEntryModel.class);
        ArrayList<AbstractOrderEntryModel> orderEntries = new ArrayList<AbstractOrderEntryModel>();
        orderEntries.add(orderEntry);
        ArrayList<PaymentTransactionModel> paymentTransactions = new ArrayList<>();
        paymentTransactions.add(transactionModel);
        ArrayList<PaymentTransactionEntryModel> paymentTransactionEntries = new ArrayList<>();
        paymentTransactionEntries.add(entryModel);

        when(transactionsDao.getTransactionEntryById(TRANSACTION_ID)).thenReturn(entryModel);
        when(entryModel.getType()).thenReturn(PaymentTransactionType.AUTHORIZATION);

        when(entryModel.getPaymentTransaction()).thenReturn(transactionModel);
        when(transactionModel.getOrder()).thenReturn(order);
        when(order.getEntries()).thenReturn(orderEntries);
        when(orderEntry.getOrder()).thenReturn(order);
        when(order.getPaymentTransactions()).thenReturn(paymentTransactions);
        when(transactionModel.getEntries()).thenReturn(paymentTransactionEntries);

        when(paypalPaymentService.getCancelableTransactions(order)).thenReturn(paymentTransactionEntries);
        when(paypalPaymentService.getNewPaymentTransactionEntryCode(transactionModel,PaymentTransactionType.CANCEL)).thenReturn("newCode");
        when(modelService.create(PaymentTransactionEntryModel.class)).thenReturn(newTransactionEntryModel);
        when(paypalPaymentService.isOrderCanceled(order)).thenReturn(true);

        unit.voidPaymentTransaction(TRANSACTION_ID);

        verify(modelService).save(entryModel);
        verify(modelService).save(order);
        verify(modelService).save(newTransactionEntryModel);
        verify(modelService).refresh(order);

    }

}
